@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2011 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -168,19 +168,29 @@ def process_cmd_line():
         help = _("preview files to be deleted and other changes"))
     parser.add_option("--preset", action="store_true",
         help = _("use options set in the graphical interface"))
+    if 'nt' == os.name:
+        parser.add_option("--update-winapp2", action="store_true",
+            help = _("update winapp2.ini, if a new version is available"))
     parser.add_option("-v", "--version", action = "store_true",
         help = _("output version information and exit"))
     parser.add_option('-o', '--overwrite', action = 'store_true',
         help = _('overwrite files to hide contents'))
     (options, args) = parser.parse_args()
+    did_something = False
     if options.version:
         print """
 BleachBit version %s
-Copyright (C) 2011 Andrew Ziem.  All rights reserved.
+Copyright (C) 2013 Andrew Ziem.  All rights reserved.
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION
         sys.exit(0)
+    if 'nt' == os.name and options.update_winapp2:
+        import Update
+        print "Checking online for updates to winapp2.ini"
+        Update.check_updates(False, True, lambda x: sys.stdout.write("%s\n" % x))
+        # updates can be combined with --list, --preview, --clean
+        did_something = True
     if options.list_cleaners:
         list_cleaners()
         sys.exit(0)
@@ -199,14 +209,15 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION
     if options.gui:
         import gtk
         import GUI
-        gui = GUI.GUI(uac=not options.no_uac)
+        GUI.GUI(uac=not options.no_uac)
         gtk.main()
         sys.exit(0)
     if options.sysinfo:
         import Diagnostic
         print Diagnostic.diagnostic_info()
         sys.exit(0)
-    parser.print_help()
+    if not did_something:
+        parser.print_help()
 
 
 
