@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -33,13 +33,11 @@ except:
     HAVE_GTK = False
 import os.path
 import re
-import subprocess
 import sys
 import traceback
 
 import Command
 import FileUtilities
-import General
 import Memory
 import Special
 
@@ -204,7 +202,6 @@ class Firefox(Cleaner):
         elif 'nt' == os.name:
             self.profile_dir = "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Profiles\\*\\"
             self.add_running('exe', 'firefox.exe')
-            self.add_running('pathname', self.profile_dir + 'parent.lock')
 
     def get_description(self):
         return _("Web browser")
@@ -225,6 +222,8 @@ class Firefox(Cleaner):
         if 'cache' == option_id:
             dirs = FileUtilities.expand_glob_join(cache_base, "Cache*")
             dirs += FileUtilities.expand_glob_join(cache_base, "OfflineCache")
+            if 'nt' == os.name:
+                dirs += FileUtilities.expand_glob_join(cache_base, "jumpListCache") # Windows 8
             for dirname in dirs:
                 for filename in children_in_directory(dirname, False):
                     yield Command.Delete(filename)
@@ -241,6 +240,7 @@ class Firefox(Cleaner):
         if 'crash_reports' == option_id:
             for filename in children_in_directory(crashdir, False):
                 files += [ filename ]
+            files += FileUtilities.expand_glob_join(self.profile_dir, "minidumps/*.dmp")
         # DOM storage
         if 'dom' == option_id:
             files += FileUtilities.expand_glob_join(self.profile_dir, "webappsstore.sqlite")
@@ -272,6 +272,9 @@ class Firefox(Cleaner):
         if 'url_history' == option_id:
             # Firefox version 1
             files += FileUtilities.expand_glob_join(self.profile_dir, "history.dat")
+            # Firefox 21 on Windows
+            if 'nt' == os.name:
+                files += FileUtilities.expand_glob_join(cache_base, "thumbnails/*.png")
             # see also function other_cleanup()
         # finish
         for filename in files:
@@ -354,13 +357,25 @@ class OpenOfficeOrg(Cleaner):
                         yield Command.Function(path, \
                             Special.delete_ooo_history,
                             _('Delete the usage history'))
-
+                # ~/.openoffice.org/3/user/registrymodifications.xcu
+                #       Apache OpenOffice.org 3.4.1 from openoffice.org on Ubuntu 13.04
+                # %AppData%\OpenOffice.org\3\user\registrymodifications.xcu
+                #       Apache OpenOffice.org 3.4.1 from openoffice.org on Windows XP
+                for path in FileUtilities.expand_glob_join(prefix, "user/registrymodifications.xcu"):
+                    if os.path.lexists(path):
+                        yield Command.Function(path, \
+                            Special.delete_office_registrymodifications,
+                            _('Delete the usage history'))
 
 class System(Cleaner):
-    """System in general"""
+    """Clean the system in general"""
 
     def __init__(self):
         Cleaner.__init__(self)
+
+        ###
+        ### options for Linux and BSD
+        ###
         if 'posix' == os.name:
             # TRANSLATORS: desktop entries are .desktop files in Linux tha
             # make up the application menu (the menu that shows BleachBit,
@@ -381,11 +396,19 @@ class System(Cleaner):
             self.add_option('rotated_logs', _('Rotated logs'), _('Delete old system logs'))
             self.add_option('recent_documents', _('Recent documents list'), _('Delete the list of recently used documents'))
             self.add_option('trash', _('Trash'), _('Empty the trash'))
+
+        ###
+        ### options just for Linux
+        ###
         if sys.platform.startswith('linux'):
             self.add_option('memory', _('Memory'), \
             # TRANSLATORS: 'free' means 'unallocated'
                _('Wipe the swap and free memory'))
             self.set_warning('memory', _('This option is experimental and may cause system problems.'))
+
+        ###
+        ### options just for Microsoft Windows
+        ###
         if 'nt' == os.name:
             self.add_option('logs', _('Logs'), _('Delete the logs'))
             self.add_option('memory_dump', _('Memory dump'), _('Delete the file memory.dmp'))
@@ -396,8 +419,19 @@ class System(Cleaner):
             # TRANSLATORS: 'Update' is a noun, and 'Update uninstallers' is an option to delete
             # the uninstallers for software updates.
             self.add_option('updates', _('Update uninstallers'), _('Delete uninstallers for Microsoft updates including hotfixes, service packs, and Internet Explorer updates'))
+
+        ###
+        ### options for GTK+
+        ###
+
         if HAVE_GTK:
             self.add_option('clipboard', _('Clipboard'), _('The desktop environment\'s clipboard used for copy and paste operations'))
+
+        ###
+        ### options common to all platforms
+        ###
+        # TRANSLATORS: "Custom" is an option allowing the user to specify which files and folders will be erased.
+        self.add_option('custom', _('Custom'), _('Delete user-specified files and folders'))
         # TRANSLATORS: 'free' means 'unallocated'
         self.add_option('free_disk_space', _('Free disk space'), \
             # TRANSLATORS: 'free' means 'unallocated'
@@ -415,13 +449,30 @@ class System(Cleaner):
         return _("System")
 
     def get_commands(self, option_id):
+        # This variable will collect fully expanded file names, and
+        # at the end of this function, they will be checked they exist
+        # and processed through Command.Delete().
+        files = []
+
         # cache
         if 'posix' == os.name and 'cache' == option_id:
             dirname = os.path.expanduser("~/.cache/")
             for filename in children_in_directory(dirname, True):
                 if self.whitelisted(filename):
                     continue
-                yield Command.Delete(filename)
+                files += [ filename ]
+
+        # custom
+        if 'custom' == option_id:
+            for (c_type, c_path) in options.get_custom_paths():
+                if 'file' == c_type:
+                    files += [ c_path ]
+                elif 'folder' == c_type:
+                    files += [ c_path ]
+                    for path in children_in_directory(c_path, True):
+                        files += [ path ]
+                else:
+                    raise RuntimeError('custom folder has invalid type %s' % c_type)
 
         # menu
         menu_dirs = [ '~/.local/share/applications', \
@@ -449,9 +500,10 @@ class System(Cleaner):
                 yield Command.Delete(path)
 
         # Windows logs
-        files = []
         if 'nt' == os.name and 'logs' == option_id:
             paths = ( \
+                '$ALLUSERSPROFILE\\Application Data\\Microsoft\\Dr Watson\\*.log', \
+                '$ALLUSERSPROFILE\\Application Data\\Microsoft\\Dr Watson\\user.dmp', \
                 '$LocalAppData\\Microsoft\\Windows\\WER\\ReportArchive\\*\\*', \
                 '$LocalAppData\\Microsoft\\Windows\WER\\ReportQueue\\*\\*', \
                 '$programdata\\Microsoft\\Windows\\WER\\ReportArchive\\*\\*', \
@@ -469,6 +521,7 @@ class System(Cleaner):
                 '$windir\\Debug\\UserMode\\*.log', \
                 '$windir\\Debug\\UserMode\\ChkAcc.bak', \
                 '$windir\\Debug\\UserMode\\userenv.bak', \
+                '$windir\\Microsoft.NET\Framework\*\*.log', \
                 '$windir\\pchealth\\helpctr\\Logs\\hcupdate.log', \
                 '$windir\\security\\logs\\*.log', \
                 '$windir\\security\\logs\\*.old', \
@@ -518,7 +571,8 @@ class System(Cleaner):
                 pathname = os.path.expanduser(pathname)
                 if os.path.lexists(pathname):
                     yield Command.Shred(pathname)
-                    gtk.RecentManager().purge_items()
+                    if HAVE_GTK:
+                        gtk.RecentManager().purge_items()
 
 
         if 'posix' == os.name and 'rotated_logs' == option_id:
@@ -572,11 +626,6 @@ class System(Cleaner):
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
 
-        # return queued files
-        for filename in files:
-            if os.path.lexists(filename):
-                yield Command.Delete(filename)
-
         # clipboard
         if HAVE_GTK and 'clipboard' == option_id:
             def clear_clipboard():
@@ -633,6 +682,11 @@ class System(Cleaner):
             for wu in Windows.delete_updates():
                 yield wu
 
+        # return queued files
+        for filename in files:
+            if os.path.lexists(filename):
+                yield Command.Delete(filename)
+
 
     def whitelisted(self, pathname):
         """Return boolean whether file is whitelisted"""
