@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -31,7 +31,7 @@ import sys
 if 'nt' == os.name:
     from win32com.shell import shell, shellcon
 
-APP_VERSION = "0.9.3"
+APP_VERSION = "0.9.6"
 APP_NAME = "BleachBit"
 APP_URL = "http://bleachbit.sourceforge.net"
 
@@ -63,13 +63,12 @@ license_filenames = ('/usr/share/common-licenses/GPL-3', # Debian, Ubuntu
     '/usr/share/doc/bleachbit-' + APP_VERSION + '/COPYING', # CentOS, Fedora, RHEL
     '/usr/share/doc/packages/bleachbit/COPYING', # OpenSUSE 11.1
     '/usr/share/doc/bleachbit/COPYING', # Mandriva
-    '/usr/pkg/share/doc/bleachbit/COPYING' ) # NetBSD 5
+    '/usr/pkg/share/doc/bleachbit/COPYING', # NetBSD 5
+    '/usr/share/licenses/common/GPL3/license.txt' ) # Arch Linux
 for lf in license_filenames:
     if os.path.exists(lf):
         license_filename = lf
         break
-if None == license_filename:
-    print 'warning: cannot find GPLv3 license text file'
 
 # configuration
 portable_mode = False
