@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2011 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -30,10 +30,8 @@ import locale
 import os
 import platform
 import sys
-import traceback
 
 if 'nt' == os.name:
-    import Windows
     from win32com.shell import shell
 
 
