@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -246,7 +246,7 @@ def delete(path, shred = False):
         else:
             os.remove(path)
     else:
-        print "info: unsupported special file type: '%s'" % (path)
+        print "info: special file type cannot be deleted: '%s'" % (path)
 
 
 def ego_owner(filename):
@@ -322,7 +322,15 @@ def getsize(path):
     """Return the actual file size considering spare files
        and symlinks"""
     if 'posix' == os.name:
-        __stat = os.lstat(path)
+        try:
+            __stat = os.lstat(path)
+        except OSError, e:
+            # OSError: [Errno 13] Permission denied
+            # can happen when a regular user is trying to find the size of /var/log/hp/tmp
+            # where /var/log/hp is 0774 and /var/log/hp/tmp is 1774
+            if errno.EACCES == e.errno:
+                return 0
+            raise
         return __stat.st_blocks * 512
     if 'nt' == os.name:
         # On rare files os.path.getsize() returns access denied, so try FindFilesW instead
@@ -392,6 +400,18 @@ def listdir(directory):
         yield os.path.join(dirname, filename)
 
 
+def sync():
+    """Flush file system buffers. sync() is different than fsync()"""
+    if 'posix' == os.name:
+        import ctypes
+        rc = ctypes.cdll.LoadLibrary('libc.so.6').sync()
+        if 0 != rc:
+            print 'ERROR: sync() returned code %d' % rc
+    if 'nt' == os.name:
+        import ctypes
+        ctypes.cdll.LoadLibrary('msvcrt.dll')._flushall()
+
+
 def whitelisted(path):
     """Check whether this path is whitelisted"""
     from Options import options
@@ -439,16 +459,62 @@ def wipe_contents(path, truncate = True):
     while size > 0:
         f.write(blanks)
         size -= 4096
-    f.flush()
+    f.flush() # flush to OS buffer
+    os.fsync(f.fileno()) # force write to disk
     if truncate:
         f.truncate(0)
         f.flush()
+        os.fsync(f.fileno())
     f.close()
 
 
+def wipe_inodes(pathname):
+    """Create empty files with long names to clear remanence in metadata"""
+    # When the file system is full, Linux 3.5 will sometimes
+    # give an error and sometimes succeed.
+    errors = 0
+    files = []
+    suffix = ' ' * (os.statvfs(pathname).f_namemax - len(pathname) - 10)
+    print 'info: creating empty files'
+    print 'debug: suffix length = ', len(suffix)
+
+    # Creating the files in a sub-directory improves recovery
+    # of disk space:
+    # http://bleachbit.sourceforge.net/news/deleting-files-doesnt-free-disk-space
+    tmpdir = os.path.join(pathname, __random_string(10))
+    os.mkdir(tmpdir)
+
+    # create empty files until file system is exhausted
+    while True:
+        try:
+            (fd, fn) = tempfile.mkstemp(dir=tmpdir, suffix=suffix, prefix = '')
+        except:
+            #print 'error', sys.exc_info()[1]
+            errors += 1
+        else:
+            errors = 0
+            files.append(fn)
+            os.close(fd)
+        if errors > 10 or 0 == os.statvfs(pathname).f_ffree:
+            break
+
+    # flush once because it is slow
+    sync()
+
+    # display effectiveness
+    stats = os.statvfs(pathname)
+    print 'info: %d inodes available, %d to super user' % (stats.f_ffree, stats.f_favail)
+
+    # clean up
+    for f in files:
+        os.remove(f)
+    os.rmdir(tmpdir)
+    print 'debug: done deleting empty files'
+
+
 def wipe_name(pathname1):
     """Wipe the original filename and return the new pathname"""
-    (head, tail) = os.path.split(pathname1)
+    (head, _) = os.path.split(pathname1)
     # reference http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
     maxlen = 226
     # first, rename to a long name
@@ -491,19 +557,19 @@ def wipe_path(pathname, idle = False ):
         i = 0
         while True:
             try:
-                f = tempfile.TemporaryFile(dir = pathname, suffix = __random_string(maxlen))
+                kwargs = { 'dir' : pathname, 'suffix' : __random_string(maxlen) }
+                if sys.hexversion >= 0x02060000:
+                    kwargs['delete'] = False
+                f = tempfile.NamedTemporaryFile(**kwargs)
                 break
             except OSError, e:
-                if e.errno == errno.ENAMETOOLONG:
-                    # OSError: [Errno ENAMETOOLONG] File name too long
-                    if maxlen > 10:
-                        maxlen -= 10
-                    i += 1
-                    if i > 100:
-                        f = tempfile.TemporaryFile(dir = pathname)
-                        break
-                else:
-                    raise
+                if e.errno in (errno.ENAMETOOLONG, errno.ENOSPC):
+                    # ext3 on Linux 3.5 returns ENOSPC if the full path is greater than 264.
+                    # Shrinking the size helps.
+                    if maxlen > 5:
+                        maxlen -= 5
+                        continue
+                raise
         return f
 
 
@@ -512,7 +578,7 @@ def wipe_path(pathname, idle = False ):
         remaining_bytes = free_space(pathname)
         done_bytes = start_free_bytes - remaining_bytes
         if done_bytes < 0:
-            # mayber user deleted large file after starting wipe
+            # maybe user deleted large file after starting wipe
             done_bytes = 0
         if 0 == start_free_bytes:
             done_percent = 0
@@ -529,9 +595,11 @@ def wipe_path(pathname, idle = False ):
     total_bytes = 0
     start_free_bytes = free_space(pathname)
     start_time = time.time()
-    # repeat to clear inodes (Linux) / MFT (Master File Table on Windows)
+    # Because FAT32 has a maximum file size of 4,294,967,295 bytes,
+    # this loop is sometimes necessary to create multiple files.
     while True:
         try:
+            print 'debug: wipe_path: creating new temporary file'
             f = temporaryfile()
         except OSError, e:
             # Linux gives errno 24
@@ -541,26 +609,26 @@ def wipe_path(pathname, idle = False ):
             else:
                 raise
         last_idle = time.time()
-        # blocks
-        blanks = chr(0) * 4096
-        try:
-            while True:
+        # Write large blocks to quickly fill the disk.
+        blanks = chr(0) * 65535
+        while True:
+            try:
                 f.write(blanks)
-                if idle and (time.time() - last_idle) > 2:
-                    # Keep the GUI responding, and allow the user to abort.
-                    # Also display the ETA.
-                    yield estimate_completion()
-                    last_idle = time.time()
-        except IOError, e:
-            if e.errno != errno.ENOSPC:
-                raise
-        # individual characters
-        try:
-            while True:
-                f.write(chr(0))
-        except IOError, e:
-            if e.errno != errno.ENOSPC:
-                raise
+            except IOError, e:
+                if e.errno == errno.ENOSPC:
+                    if len(blanks) > 1:
+                        # Try writing smaller blocks
+                        blanks = blanks[0 : (len(blanks)/2)]
+                    else:
+                        break
+                elif e.errno != errno.EFBIG:
+                    raise
+            if idle and (time.time() - last_idle) > 2:
+                # Keep the GUI responding, and allow the user to abort.
+                # Also display the ETA.
+                yield estimate_completion()
+                last_idle = time.time()
+        # Write to OS buffer
         try:
             f.flush()
         except:
@@ -568,18 +636,46 @@ def wipe_path(pathname, idle = False ):
             # seen on Microsoft Windows XP SP3 with ~30GB free space but
             # not on another XP SP3 with 64MB free space
             print "info: exception on f.flush()"
+        os.fsync(f.fileno()) # write to disk
+        # Remember to delete
         files.append(f)
+        # For statistics
         total_bytes += f.tell()
+        # If no bytes were written, then quit
+        if f.tell() < 1:
+            break
+    # sync to disk
+    sync()
     # statistics
     elapsed_sec = time.time() - start_time
     rate_mbs = (total_bytes / (1000*1000) ) / elapsed_sec
-    print 'debug: wrote %d files and %d bytes in %d seconds at %.2f MB/s' % \
+    print 'note: wrote %d files and %d bytes in %d seconds at %.2f MB/s' % \
         (len(files), total_bytes, elapsed_sec, rate_mbs)
+    # how much free space is left (should be near zero)
+    if 'posix' == os.name:
+        stats = os.statvfs(pathname)
+        print 'note: %d bytes and %d inodes available to non-super-user' % (stats.f_bsize * stats.f_bavail, stats.f_favail)
+        print 'note: %d bytes and %d inodes available to super-user' % (stats.f_bsize * stats.f_bfree, stats.f_ffree)
     # truncate and close files
     for f in files:
         f.truncate(0)
-        f.close()
-    # files are removed implicitly
+
+        while True:
+            try:
+                # Nikita: I noticed a bug that prevented file handles from
+                # being closed on FAT32. It sometimes takes two .close() calls
+                # to do actually close (and therefore delete) a temporary file
+                f.close()
+                break
+            except IOError, e:
+                if e.errno == 0:
+                    print 'debug: handled unknown error 0'
+                    time.sleep(0.1)
+        # explicitly delete
+        delete(f.name)
+    # quickly wipe inodes
+    if 'posix' == os.name and os.statvfs(pathname).f_ffree > 0:
+        wipe_inodes(pathname)
 
 
 def vacuum_sqlite3(path):
