@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -98,7 +98,7 @@ class TreeInfoModel:
             c_value = options.get_tree(c_id, None)
             if not c_value and options.get('auto_hide') \
                 and backends[key].auto_hide():
-                print "info: automatically hiding cleaner '%s'" % (c_id)
+                #print "info: automatically hiding cleaner '%s'" % (c_id)
                 continue
             parent = self.tree_store.append(None, (c_name, c_value, c_id))
             for (o_id, o_name) in backends[key].get_options():
@@ -396,7 +396,7 @@ class GUI:
                 link: GuiBasic.open_url(link, self.window, False))
         dialog = gtk.AboutDialog()
         dialog.set_comments(_("Program to clean unnecessary files"))
-        dialog.set_copyright("Copyright (C) 2012 Andrew Ziem")
+        dialog.set_copyright("Copyright (C) 2013 Andrew Ziem")
         try:
             dialog.set_license(open(license_filename).read())
         except:
@@ -421,11 +421,11 @@ class GUI:
         """Show diagnostic information"""
         dialog = gtk.Dialog(_("System information"), parent)
         dialog.resize(600, 400)
-        buffer = gtk.TextBuffer()
+        txtbuffer = gtk.TextBuffer()
         import Diagnostic
         txt = Diagnostic.diagnostic_info()
-        buffer.set_text(txt)
-        textview = gtk.TextView(buffer)
+        txtbuffer.set_text(txt)
+        textview = gtk.TextView(txtbuffer)
         textview.set_editable(False)
         swindow = gtk.ScrolledWindow()
         swindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -571,7 +571,6 @@ class GUI:
         # find the seleted option
         model = treeview.get_model()
         option_id = model[path][2]
-        option_name = model[path][0]
         cleaner_id = model[path[0]][2]
         cleaner_name = model[path[0]][0]
         # make a menu
@@ -604,11 +603,11 @@ class GUI:
             raise RuntimeError('unexpected type: ' + str(type(status)))
 
 
-    def update_total_size(self, bytes):
+    def update_total_size(self, bytes_removed):
         """Callback to update the total size cleaned"""
         context_id = self.status_bar.get_context_id('size')
-        text = FileUtilities.bytes_to_human(bytes)
-        if 0 == bytes:
+        text = FileUtilities.bytes_to_human(bytes_removed)
+        if 0 == bytes_removed:
             text = ""
         self.status_bar.push(context_id, text)
 
