@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -26,7 +26,6 @@ Wipe memory
 
 
 
-import ctypes
 import os
 import re
 import subprocess
@@ -48,6 +47,29 @@ def count_swap_linux():
     return count
 
 
+def get_proc_swaps():
+    """Return the output of 'swapon -s'"""
+    # Usually 'swapon -s' is identical to '/proc/swaps'
+    # Here is one exception: https://bugs.launchpad.net/ubuntu/+source/bleachbit/+bug/1092792
+    (rc, stdout, _) = General.run_external(['swapon', '-s'])
+    if 0 == rc:
+        return stdout
+    print 'debug: "swapoff -s" failed so falling back to /proc/swaps'
+    return open("/proc/swaps").read()
+
+
+def parse_swapoff(swapoff):
+    """Parse the output of swapoff and return the device name"""
+    # English is 'swapoff on /dev/sda5' but German is 'swapoff für ...'
+    # Example output in English with LVM and hyphen: 'swapoff on /dev/mapper/lubuntu-swap_1'
+    # This matches swap devices and swap files
+    ret = re.search('^swapoff .* (/[\w/\.-]+)$', swapoff)
+    if not ret:
+        # no matches
+        return None
+    return ret.group(1)
+
+
 def disable_swap_linux():
     """Disable Linux swap and return list of devices"""
     if 0 == count_swap_linux():
@@ -62,13 +84,11 @@ def disable_swap_linux():
         line = line.replace('\n', '')
         if '' == line:
             continue
-        # English is 'swapoff on /dev/sda5' but German is 'swapoff für ...'
-        # This matches swap devices and swap files
-        ret = re.search('^swapoff .* (/[\w/\.]+)$', line)
+        ret = parse_swapoff(line)
         if None == ret:
             raise RuntimeError("Unexpected output:\nargs='%(args)s'\nstdout='%(stdout)s'\nstderr='%(stderr)s'" \
                 % { 'args' : str(args), 'stdout' : stdout, 'stderr' : stderr } )
-        devices.append(ret.group(1))
+        devices.append(ret)
     return devices
 
 
@@ -83,17 +103,18 @@ def enable_swap_linux():
         raise RuntimeError(outputs[1].replace("\n", ""))
 
 
-def fill_memory_linux():
-    """Fill unallocated memory"""
-    # make self primary target for Linux out-of-memory killer
-    path = '/proc/%d/oomadj' % os.getpid()
+def make_self_oom_target_linux():
+    """Make the current process the primary target for Linux out-of-memory killer"""
+    # In Linux 2.6.36 the system changed from oom_adj to oom_score_adj
+    path = '/proc/%d/oom_score_adj' % os.getpid()
     if os.path.exists(path):
-        f = open(path)
-        f.write('15')
-        f.close()
+        open(path, 'w').write('1000')
+    else:
+        path = '/proc/%d/oomadj' % os.getpid()
+        if os.path.exists(path):
+            open(path, 'w').write('15')
     # OOM likes nice processes
     print 'debug: new nice value', os.nice(19)
-    libc = ctypes.cdll.LoadLibrary("libc.so.6")
     # OOM prefers non-privileged processes
     try:
         uid = General.getrealuid()
@@ -103,34 +124,34 @@ def fill_memory_linux():
             os.seteuid(uid)
     except:
         traceback.print_exc()
-    # fill memory
-    def fill_helper():
-        report_free()
-        allocbytes = int(physical_free() * 0.75)
-        if allocbytes < 1024:
-            return
-        megabytes = allocbytes / (1024**2)
-        print "info: allocating %.2f MB (%d B) memory" % (megabytes, allocbytes)
-        mbuffer = libc.malloc(allocbytes)
-        if 0 == mbuffer:
-            print 'debug: malloc() returned', mbuffer
-            return
-        print "debug: wiping %.2f MB I just allocated" % megabytes
-        libc.memset(mbuffer, 0x00, allocbytes)
-        fill_helper()
+
+
+def fill_memory_linux():
+    """Fill unallocated memory"""
+    report_free()
+    allocbytes = int(physical_free() * 0.4)
+    if allocbytes < 1024:
+        return
+    megabytes = allocbytes / (1024**2)
+    print "info: allocating and wiping %.2f MB (%d B) memory" % (megabytes, allocbytes)
+    try:
+        buf = '\x00' * allocbytes
+    except MemoryError:
+        pass
+    else:
+        fill_memory_linux()
         print "debug: freeing %.2f MB memory" % megabytes
-        libc.free(mbuffer)
-        report_free()
-    fill_helper()
+        del buf
+    report_free()
 
 
 def get_swap_size_linux(device, proc_swaps = None):
     """Return the size of the partition in bytes"""
     if None == proc_swaps:
-        proc_swaps = open("/proc/swaps").read()
+        proc_swaps = get_proc_swaps()
     line = proc_swaps.split('\n')[0]
     if not re.search('Filename\s+Type\s+Size', line):
-        raise RuntimeError("Unexpected first line in /proc/swaps '%s'" % line)
+        raise RuntimeError("Unexpected first line in swap summary '%s'" % line)
     for line in proc_swaps.split('\n')[1:]:
         ret = re.search("%s\s+\w+\s+([0-9]+)\s" % device, line)
         if ret:
@@ -143,12 +164,12 @@ def get_swap_uuid(device):
     """Find the UUID for the swap device"""
     uuid = None
     args = ['blkid', device, '-s', 'UUID']
-    (rc, stdout, stderr) = General.run_external(args)
+    (_, stdout, _) = General.run_external(args)
     for line in stdout.split('\n'):
         # example: /dev/sda5: UUID="ee0e85f6-6e5c-42b9-902f-776531938bbf"
         ret = re.search("^%s: UUID=\"([a-z0-9-]+)\"" % device, line)
         if None != ret:
-             uuid = ret.group(1)
+            uuid = ret.group(1)
     print "debug: uuid(%s)='%s'" % (device, uuid)
     return uuid
 
@@ -156,15 +177,15 @@ def get_swap_uuid(device):
 def physical_free_linux():
     """Return the physical free memory on Linux"""
     f = open("/proc/meminfo")
-    bytes = 0
+    free_bytes = 0
     for line in f:
         line = line.replace("\n","")
         ret = re.search('(MemFree|Cached):[ ]*([0-9]*) kB', line)
         if None != ret:
             kb = int(ret.group(2))
-            bytes += kb * 1024
-    if bytes > 0:
-        return bytes
+            free_bytes += kb * 1024
+    if free_bytes > 0:
+        return free_bytes
     else:
         raise Exception("unknown")
 
@@ -234,7 +255,7 @@ def wipe_swap_linux(devices, proc_swaps):
         if uuid:
             args.append("-U")
             args.append(uuid)
-        (rc, stdout, stderr) = General.run_external(args)
+        (rc, _, stderr) = General.run_external(args)
         if 0 != rc:
             raise RuntimeError(stderr.replace("\n", ""))
 
@@ -242,7 +263,7 @@ def wipe_swap_linux(devices, proc_swaps):
 def wipe_memory():
     """Wipe unallocated memory"""
     # cache the file because 'swapoff' changes it
-    proc_swaps = open("/proc/swaps").read()
+    proc_swaps = get_proc_swaps()
     devices = disable_swap_linux()
     yield True # process GTK+ idle loop
     print 'debug: detected swap devices:', devices
@@ -250,6 +271,7 @@ def wipe_memory():
     yield True
     child_pid = os.fork()
     if 0 == child_pid:
+        make_self_oom_target_linux()
         fill_memory_linux()
         sys.exit(0)
     else:
