@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -19,35 +19,55 @@
 
 
 """
-Store and retreieve user preferences
+Store and retrieve user preferences
 """
 
 
 import os
+import re
 import traceback
 import ConfigParser
 
 import Common
 import General
 
+if 'nt' == os.name:
+    from win32file import GetLongPathName
+
 
 boolean_keys = ['auto_hide', 'auto_start', 'check_beta', 'check_online_updates', 'first_start', 'shred']
 if 'nt' == os.name:
     boolean_keys.append('update_winapp2')
 
 
+def path_to_option(pathname):
+    """Change a pathname to a .ini option name (a key)"""
+    # On Windows change to lowercase and use backwards slashes.
+    pathname = os.path.normcase(pathname)
+    # On Windows expand DOS-8.3-style pathnames.
+    if 'nt' == os.name and os.path.exists(pathname):
+        pathname = GetLongPathName(pathname)
+    if ':' == pathname[1] :
+        # ConfigParser treats colons in a special way
+        pathname = pathname[0] + pathname[2:]
+    return pathname
+
 
 class Options:
     """Store and retrieve user preferences"""
 
 
     def __init__(self):
+        self.purged = False
         self.config = ConfigParser.SafeConfigParser()
+        self.config.optionxform = str # make keys case sensitive for hashpath purging
         self.restore()
 
 
     def __flush(self):
         """Write information to disk"""
+        if not self.purged:
+            self.__purge()
         if not os.path.exists(Common.options_dir):
             General.makedirs(Common.options_dir)
         mkfile = not os.path.exists(Common.options_file)
@@ -56,7 +76,8 @@ class Options:
             self.config.write(_file)
         except IOError, e:
             print e
-            if 28 == e.errno:
+            from errno import ENOSPC
+            if e.errno == ENOSPC:
                 print "Error: disk is full writing configuration '%s'" % Common.options_file
             else:
                 raise
@@ -64,6 +85,21 @@ class Options:
             General.chownself(Common.options_file)
 
 
+    def __purge(self):
+        """Clear out obsolete data"""
+        self.purged = True
+        if not self.config.has_section('hashpath'):
+            return
+        for option in self.config.options('hashpath'):
+            pathname = option
+            if 'nt' == os.name and re.search('^[a-z]\\\\', option):
+                # restore colon lost because ConfigParser treats colon special in keys
+                pathname = pathname[0] + ':' + pathname[1:]
+            if not os.path.lexists(pathname):
+                # the file does not on exist, so forget it
+                self.config.remove_option('hashpath', option)
+
+
     def __set_default(self, key, value):
         """Set the default value"""
         if not self.config.has_option('bleachbit', key):
@@ -81,6 +117,11 @@ class Options:
         return self.config.get(section, option)
 
 
+    def get_hashpath(self, pathname):
+        """Recall the hash for a file"""
+        return self.get(path_to_option(pathname), 'hashpath')
+
+
     def get_language(self, langid):
         """Retrieve value for whether to preserve the language"""
         if not self.config.has_option('preserve_languages', langid):
@@ -105,9 +146,8 @@ class Options:
             values.append(self.config.get(section, option))
         return values
 
-    def get_whitelist_paths(self):
-        """Return the whitelist of paths"""
-        section = "whitelist/paths"
+    def get_paths(self, section):
+        """Abstracts get_whitelist_paths and get_custom_paths"""
         if not self.config.has_section(section):
             return []
         myoptions = []
@@ -118,12 +158,22 @@ class Options:
             myoptions.append(option[0:pos])
         values = []
         for option in set(myoptions):
-            wl_type = self.config.get(section, option + '_type')
-            wl_path = self.config.get(section, option + '_path')
-            values.append( ( wl_type, wl_path ) )
+            p_type = self.config.get(section, option + '_type')
+            p_path = self.config.get(section, option + '_path')
+            values.append( ( p_type, p_path ) )
         return values
 
 
+    def get_whitelist_paths(self):
+        """Return the whitelist of paths"""
+        return self.get_paths("whitelist/paths")
+
+
+    def get_custom_paths(self):
+        """Return list of custom paths"""
+        return self.get_paths("custom/paths")
+
+
     def get_tree(self, parent, child):
         """Retrieve an option for the tree view.  The child may be None."""
         option = parent
@@ -186,13 +236,16 @@ class Options:
 
     def set(self, key, value, section = 'bleachbit', commit = True):
         """Set a general option"""
-        if section == 'hashpath' and key[1] == ':':
-            key = key[0] + key[2:]
         self.config.set(section, key, str(value))
         if commit:
             self.__flush()
 
 
+    def set_hashpath(self, pathname, hashvalue):
+        """Remember the hash of a path"""
+        self.set(path_to_option(pathname), hashvalue, 'hashpath')
+
+
     def set_list(self, key, values):
         """Set a value which is a list data type"""
         section = "list/%s" % key
@@ -207,7 +260,7 @@ class Options:
 
 
     def set_whitelist_paths(self, values):
-        """Save thee whitelist"""
+        """Save the whitelist"""
         section = "whitelist/paths"
         if self.config.has_section(section):
             self.config.remove_section(section)
@@ -220,6 +273,20 @@ class Options:
         self.__flush()
 
 
+    def set_custom_paths(self, values):
+        """Save the customlist"""
+        section = "custom/paths"
+        if self.config.has_section(section):
+            self.config.remove_section(section)
+        self.config.add_section(section)
+        counter = 0
+        for value in values:
+            self.config.set(section, str(counter) + '_type', value[0])
+            self.config.set(section, str(counter) + '_path', value[1])
+            counter += 1
+        self.__flush()
+
+
     def set_language(self, langid, value):
         """Set the value for a locale (whether to preserve it)"""
         if not self.config.has_section('preserve_languages'):
