@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -88,9 +88,11 @@ def delete_chrome_favicons(path):
     ver = __get_chrome_history(path)
     cmds = ""
 
-    if ver in [4, 20]:
+    if ver in [4, 20, 22, 23, 25]:
         # Version 4 includes Chromium 12
-        # Version 20 includes Chromium 14, Google Chome 15
+        # Version 20 includes Chromium 14, Google Chrome 15, Google Chrome 19
+        # Version 22 includes Google Chrome 20
+        # Version 25 is Google Chrome 26
 
         # icon_mapping
         cols = ('page_url',)
@@ -147,16 +149,17 @@ def delete_chrome_keywords(path):
     cmds  = __shred_sqlite_char_columns('keywords', cols, where)
     cmds += "update keywords set usage_count = 0;"
     ver = __get_chrome_history(path, 'Web Data')
-    if ver >= 43:
+    if ver >= 43 and ver < 49:
         # keywords_backup table first seen in Google Chrome 17 / Chromium 17 which is Web Data version 43
+        # In Google Chrome 25, the table is gone.
         cmds += __shred_sqlite_char_columns('keywords_backup', cols, where)
         cmds += "update keywords_backup set usage_count = 0;"
 
     FileUtilities.execute_sqlite3(path, cmds)
 
 
-def delete_libreoffice_history(path):
-    """Erase LibreOffice's MRU in registrymodifications.xcu"""
+def delete_office_registrymodifications(path):
+    """Erase LibreOffice 3.4 and Apache OpenOffice.org 3.4 MRU in registrymodifications.xcu"""
     import xml.dom.minidom
     dom1 = xml.dom.minidom.parse(path)
     modified = False
@@ -217,26 +220,31 @@ def delete_mozilla_url_history(path):
 
 
 def delete_ooo_history(path):
-    """Erase the OpenOffice.org MRU in Common.xcu"""
+    """Erase the OpenOffice.org MRU in Common.xcu.  No longer valid in Apache OpenOffice.org 3.4."""
     import xml.dom.minidom
     dom1 = xml.dom.minidom.parse(path)
+    changed = False
     for node in dom1.getElementsByTagName("node"):
         if node.hasAttribute("oor:name"):
             if "History" == node.getAttribute("oor:name"):
                 node.parentNode.removeChild(node)
                 node.unlink()
+                changed = True
                 break
-    dom1.writexml(open(path, "w"))
+    if changed:
+        dom1.writexml(open(path, "w"))
 
 
 def get_chrome_bookmark_ids(history_path):
     """Given the path of a history file, return the ids in the
     urls table that are bookmarks"""
     bookmark_path = os.path.join(os.path.dirname(history_path), 'Bookmarks')
+    if not os.path.exists(bookmark_path):
+        return []
     urls = get_chrome_bookmark_urls(bookmark_path)
     ids = []
     for url in urls:
-       ids +=  get_sqlite_int(history_path, 'select id from urls where url=?', (url,))
+        ids +=  get_sqlite_int(history_path, 'select id from urls where url=?', (url,))
     return ids
 
 
@@ -250,6 +258,24 @@ def get_chrome_bookmark_urls(path):
     # read file to parser
     js = json.load(open(path, 'r'))
 
-    # find bookmarks
-    return [bookmark['url'] for bookmark in js['roots']['bookmark_bar']['children']]
+    # empty list
+    urls = [ ]
 
+    # local recursive function
+    def get_chrome_bookmark_urls_helper(node):
+        if not isinstance(node, dict):
+            return
+        if not node.has_key('type'):
+            return
+        if node['type'] == "folder":
+            # folders have children
+            for child in node['children']:
+                get_chrome_bookmark_urls_helper(child)
+        if node['type'] == "url" and node.has_key('url'):
+            urls.append(node['url'])
+
+    # find bookmarks
+    for node in js['roots']:
+        get_chrome_bookmark_urls_helper(js['roots'][node])
+
+    return list(set(urls)) # unique
