@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2012 Andrew Ziem
+## Copyright (C) 2013 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -96,11 +96,8 @@ class UpdateTestCase(unittest.TestCase):
         if os.path.exists(fn):
             print 'note: deleting %s' % fn
             os.unlink(fn)
-        if os.path.exists(personal_cleaners_dir):
-            print 'note: deleting %s' % personal_cleaners_dir
-            os.rmdir(personal_cleaners_dir)
 
-        url = 'http://katana.oooninja.com/bleachbit/winapp2.ini'
+        url = 'http://www.winapp2.com/Winapp2.ini'
 
         def append_text(s):
             print s
